from pyspark.sql.functions import *
from pyspark.sql.functions import udf
from pyspark.sql.types import StringType
import re
from pyspark.sql import SparkSession
from pyspark.sql.types import LongType

ss = SparkSession.getActiveSession()

def suppressLeadingZeroes(x):
    """
       Returns the string after removing zeroes from the left of the string.
       Use the suppressLeadingZeroesUDF. 
    
       Parameters:
       String column

       Examples:
       spark.sql("select suppressLeadingZeroesUDF(columnname) from TABLE").show() to use as a query.
       df=spark.sql("select suppressLeadingZeroesUDF(columnname) from TABLE") to store the values in a DataFrame
    """
    s=str(x)
    match = re.search("^(0+)(\d+)$", s)
    output = match.group(2) if match else s
    return output

def padLeadingZeroes(max_length,x):
    """
       Adds zeroes to the left of string if the length of string is less than the maximum length.
        Use the padZeroUDF. 
        
       Parameters:
       Max_length
       Column of Numeric Strings or Numbers
       
       Examples:
       spark.sql("select padZeroUDF(Max_Length,columnname) from TABLE").show() to use as a query.
       df=spark.sql("select padZeroUDF(Max_Length,columnname) from TABLE") to store the values in a DataFrame
    """
    str1=str(x)
    output = '0'*(max_length-len(str1)) + str1
    return output

def isNumeric(x) :
    """
       Checks if the string is numeric.
       Use the isNumericUDF. 
        
       Parameters:
       Column of Strings
       
       Examples:
       spark.sql("select isNumericUDF(columnname) from TABLE").show() to use as a query.
       df=spark.sql("select isNumericUDF(columnname) from TABLE") to store the values in a DataFrame
    """
    return x.isnumeric()

from pyspark.sql.functions import round, col
from pyspark.sql.functions import split
import pyspark    

from pyspark.sql.functions import *
from pyspark.sql.functions import udf
from pyspark.sql.types import StringType
import re
from pyspark.sql import SparkSession
from pyspark.sql.types import LongType
from pyspark.sql.types import StructType

import math

def getMantissaValue(x):
    '''
        Get Mantissa Value of a decimal number or a numeric string.
        Use the getMantissaValueUDF. 
        
        Parameters:
        Column of Numeric Strings or Decimal Numbers
        
        Examples: 
        spark.sql("select ID, getMantissaValueUDF(columnname) from TABLE").show() to use as a query.
        df=spark.sql("select ID, getMantissaValueUDF(columnname) from TABLE") to store the values in a DataFrame
    '''
    s=str(x)
    g=s.split(".")
    return g[0]

def getDecimalValue(x):
    '''
        Get Decimal Value of a decimal number or a numeric string.
        Use the getDecimalValueUDF. 
        
        Parameters:
        Column of Numeric Strings or Decimal Numbers
        
        Examples: 
        spark.sql("select getDecimalValueUDF(columnname) from TABLE").show() to use as a query.
        df=spark.sql("select getDecimalValueUDF(columnname) from TABLE") to store the values in a DataFrame
    '''
    s=str(x)
    g=s.split(".")
    return g[1]

def getPrecision(x):
    '''
        Get Precision of a decimal number or a numeric string.
        Use the getPrecisionUDF. 
        
        Parameters:
        Column of Numeric Strings or Decimal Numbers
        
        Examples: 
        spark.sql("select getPrecisionUDF(columnname) from TABLE").show() to use as a query.
        df=spark.sql("select getPrecisionUDF(columnname) from TABLE") to store the values in a DataFrame
    '''
    s=str(x)
    m=getDecimalValue(s)
    return len(m)
    
def matchRegex(x,patterns):
    '''
        Check if the column of the Table contains given patterns.
        
        Parameters:
        Column: Provide Column of the Table
        patterns: Write all patterns with delimeter '**' . Example: "ar**la"
    
        
        Examples:
        spark.sql("select matchPatternUDF(columnname) from TABLE").show() to use as a query.
        df=spark.sql("select matchPatternUDF(columnname) from TABLE") to store the values in a DataFrame.
    '''
    import re
    split_pattern = patterns.split('**')
    bool_matched = ""
    for i in split_pattern:
        if re.search(i,x):
            bool_matched += "T"
        else:
            bool_matched += "F"

    return bool_matched


def valueCheck(var,listValues):
    '''
        Check whether values of a column exist in a given list.
        Use the valueCheckUDF. 
        
        Parameters:
        Column: Provide Column of the Table
        List: Provide a list of values. Examples: ('Value1','Value2')
        
        Examples: 
        spark.sql("select valueCheckUDF(columnname,list) from TABLE").show() to use as a query.
        df=spark.sql("select valueCheckUDF(columnname,list) from TABLE") to store the values in a DataFrame.
    '''
    for i in listValues:
        if(var in i):
            return 'True'
    return 'False'
    
ss = SparkSession.getActiveSession()

def suppressLeadingZeroes(x):
    """
       Returns the string after removing zeroes from the left of the string.
       Use the suppressLeadingZeroesUDF. 
    
       Parameters:
       String column

       Examples:
       spark.sql("select suppressLeadingZeroesUDF(columnname) from TABLE").show() to use as a query.
       df=spark.sql("select suppressLeadingZeroesUDF(columnname) from TABLE") to store the values in a DataFrame
    """
    s=str(x)
    match = re.search("^(0+)(\d+)$", s)
    output = match.group(2) if match else s
    return output

def padLeadingZeroes(max_length,x):
    """
       Adds zeroes to the left of string if the length of string is less than the maximum length.
        Use the padZeroUDF. 
        
       Parameters:
       Max_length
       Column of Numeric Strings or Numbers
       
       Examples:
       spark.sql("select padZeroUDF(Max_Length,columnname) from TABLE").show() to use as a query.
       df=spark.sql("select padZeroUDF(Max_Length,columnname) from TABLE") to store the values in a DataFrame
    """
    str1=str(x)
    output = '0'*(max_length-len(str1)) + str1
    return output

def isNumeric(x) :
    """
       Checks if the string is numeric.
       Use the isNumericUDF. 
        
       Parameters:
       Column of Strings
       
       Examples:
       spark.sql("select isNumericUDF(columnname) from TABLE").show() to use as a query.
       df=spark.sql("select isNumericUDF(columnname) from TABLE") to store the values in a DataFrame
    """
    return x.isnumeric()

from pyspark.sql.functions import round, col
from pyspark.sql.functions import split
import pyspark

from pyspark.sql.functions import *
from pyspark.sql.functions import udf
from pyspark.sql.types import StringType
import re
from pyspark.sql import SparkSession
from pyspark.sql.types import LongType
from pyspark.sql.types import StructType

import math

def getMantissaValue(x):
    '''
        Get Mantissa Value of a decimal number or a numeric string.
        Use the getMantissaValueUDF. 
        
        Parameters:
        Column of Numeric Strings or Decimal Numbers
        
        Examples: 
        spark.sql("select ID, getMantissaValueUDF(columnname) from TABLE").show() to use as a query.
        df=spark.sql("select ID, getMantissaValueUDF(columnname) from TABLE") to store the values in a DataFrame
    '''
    s=str(x)
    g=s.split(".")
    return g[0]

def getDecimalValue(x):
    '''
        Get Decimal Value of a decimal number or a numeric string.
        Use the getDecimalValueUDF. 
        
        Parameters:
        Column of Numeric Strings or Decimal Numbers
        
        Examples: 
        spark.sql("select getDecimalValueUDF(columnname) from TABLE").show() to use as a query.
        df=spark.sql("select getDecimalValueUDF(columnname) from TABLE") to store the values in a DataFrame
    '''
    s=str(x)
    g=s.split(".")
    return g[1]

def getPrecision(x):
    '''
        Get Precision of a decimal number or a numeric string.
        Use the getPrecisionUDF. 
        
        Parameters:
        Column of Numeric Strings or Decimal Numbers
        
        Examples: 
        spark.sql("select getPrecisionUDF(columnname) from TABLE").show() to use as a query.
        df=spark.sql("select getPrecisionUDF(columnname) from TABLE") to store the values in a DataFrame
    '''
    s=str(x)
    m=getDecimalValue(s)
    return len(m)
    
def matchRegex(x,patterns):
    '''
        Check if the column of the Table contains given patterns.
        
        Parameters:
        Column: Provide Column of the Table
        patterns: Write all patterns with delimeter '**' . Example: "ar**la"
    
        
        Examples:
        spark.sql("select matchPatternUDF(columnname) from TABLE").show() to use as a query.
        df=spark.sql("select matchPatternUDF(columnname) from TABLE") to store the values in a DataFrame.
    '''
    import re
    split_pattern = patterns.split('**')
    bool_matched = ""
    for i in split_pattern:
        if re.search(i,x):
            bool_matched += "T"
        else:
            bool_matched += "F"

    return bool_matched


def valueCheck(var,listValues):
    '''
        Check whether values of a column exist in a given list.
        Use the valueCheckUDF. 
        
        Parameters:
        Column: Provide Column of the Table
        List: Provide a list of values. Examples: ('Value1','Value2')
        
        Examples: 
        spark.sql("select valueCheckUDF(columnname,list) from TABLE").show() to use as a query.
        df=spark.sql("select valueCheckUDF(columnname,list) from TABLE") to store the values in a DataFrame.
    '''
    for i in listValues:
        if(var in i):
            return 'True'
    return 'False'
    
def cons(ss):
    ss.udf.register('suppressLeadingZeroesUDF', suppressLeadingZeroes)
    suppressLeadingZeroesUDF=udf(lambda x: suppressLeadingZeroes(x), StringType())
    
    ss.udf.register('padZeroUDF', padLeadingZeroes)
    padZeroUDF = udf(lambda x,y: padLeadingZeroes(x,y), StringType())
    
    ss.udf.register('isNumericUDF', isNumeric)
    isNumericUDF = udf(lambda x: isNumeric(x), StringType())
    
    ss.udf.register('getMantissaValueUDF', getMantissaValue)
    getMantissaValueUDF = udf(lambda x: getMantissaValue(x), StringType())

    ss.udf.register('getDecimalValueUDF', getDecimalValue)
    getDecimalValueUDF = udf(lambda x: getDecimalValue(x), StructType())

    ss.udf.register('matchPatternUDF', matchRegex)
    matchPatternUDF = udf(lambda x,y :matchRegex(x,y), StringType())

    ss.udf.register('valueCheckUDF', valueCheck)
    valueCheckUDF = udf(lambda x,y: valueCheck(x,y), StringType())

    ss.udf.register('getPrecisionUDF', getPrecision)
    getPrecisionUDF = udf(lambda x: getPrecision(x), StringType())